#!/bin/sh

HA_PREFIX="/var/packages/HighAvailability/target"
#shellcheck source-path=SCRIPTDIR source=rc.subr
. $HA_PREFIX/etc/rc.subr
lsyncdInstance=${1:?"Missing instance name"}
SYNOLSYNC="$HA_PREFIX/sbin/synolsync"
hasErr=$($SYNOLSYNC --get-status "$lsyncdInstance" | grep "error" | cut -d ":" -f2)
isWait=$($SYNOLSYNC --get-status "$lsyncdInstance" | grep "wait" | cut -d ":" -f2)
retry=0
retryMax=60   #total wait 60*2 s
hasRemote=$(HAHasRemote)

if [ 0 = "$hasErr" ] && [ 0 = "$isWait" ]; then
	while :;
	do
		if [ -z "$hasRemote" ]; then
			synoha_log notice "Remote is not online. Skip waiting lsyncd instance $lsyncdInstance"
			break
		elif synoha --is-safemode; then
			synoha_log notice "Split-brain occurred. Skip waiting lsyncd instance $lsyncdInstance"
			break
		fi
		if [ "0" != "$($SYNOLSYNC --get-events "$lsyncdInstance" | head -n 1)" ]; then
			retry=$(( retry + 1 ))
			if [ $retry -gt $retryMax ]; then
				synoha_log notice "Timed-out on waiting lsyncd instance $lsyncdInstance"
				break
			fi
			synoha_log notice "Waiting lsyncd instance $lsyncdInstance. Retry $retry/$retryMax"
			sleep 2 # delay 2s , since lsync's event aggregation time is 2s
		else
			break
		fi
	done
fi

