#!/bin/bash

HA_PREFIX="/var/packages/HighAvailability/target"
PATH=/sbin:/bin:/usr/sbin:/usr/bin:/usr/syno/sbin:/usr/syno/bin:$HA_PREFIX/sbin:$HA_PREFIX/usr/bin
SYNOHA_BIN=/var/packages/HighAvailability/target/sbin/synoha
C_LOGGER=/usr/bin/logger
BOTH_PASSIVE_REBOOT=/var/lib/ha/.both_passive_reboot
ROLE_ACTIVE=Active
ROLE_PASSIVE=Passive

to_upper()
{
	echo -n "$1" | tr "[:lower:]" "[:upper:]"
}

ha_log()
{
	level=${1:-}
	$C_LOGGER -p "user.$level" -t "$0[$$]" "[HA-$(to_upper "$level")] ${2:-}"
}

watch_both_passive()
{
	MAX_BOTH_PASSIVE_TIME=30
	both_passive_and_local_no_constraint=false
	while true ; do
		local_role=$($SYNOHA_BIN --local-role)
		remote_role=$($SYNOHA_BIN --remote-role)
		sleep=10

		if [ "$ROLE_ACTIVE" = "$local_role" ] || [ "$ROLE_ACTIVE" = "$remote_role" ]; then
			if [ -f $BOTH_PASSIVE_REBOOT ]; then
				ha_log notice "Someone save the world."
				rm $BOTH_PASSIVE_REBOOT
			fi
		fi

		if [ "$ROLE_PASSIVE" != "$local_role" ] || [ "$ROLE_PASSIVE" != "$remote_role" ]; then
			both_passive_and_local_no_constraint=false
		else
			if $SYNOHA_BIN --local-has-constraint ; then
				if $SYNOHA_BIN --remote-has-constraint ; then
					ha_log warn "Both are passive and with constraints, delete RULE_SPACE_SYNC to unleash one of them"
					crm configure delete RULE_SPACE_SYNC
				fi
				both_passive_and_local_no_constraint=false
			else
				if ! $both_passive_and_local_no_constraint ; then
					both_passive_and_local_no_constraint=true
					watch_start_time=$(date +%s)
				else
					now=$(date +%s)
					if  [ $(( now - watch_start_time )) -gt $MAX_BOTH_PASSIVE_TIME ] && [ ! -f $BOTH_PASSIVE_REBOOT ];
					then
						ha_log warn "I don't have constraint when both are passive and wait for $MAX_BOTH_PASSIVE_TIME seconds since $(date --date=@"$watch_start_time")."
						ha_log warn "Reboot and save the world."
						touch $BOTH_PASSIVE_REBOOT
						return
					fi
				fi
				sleep=1
			fi
		fi

		sleep  $sleep
	done
}

watch_opensource_daemon()
{
	declare -A daemon_pid
	set -- pacemakerd stonithd cib attrd pengine crmd lrmd corosync

	ha_log notice "Watch $*"

	# initialize
	for d ; do
		until [ -n "${daemon_pid[$d]}" ]; do
			pid=$(pidof "$d")
			if [ "$(echo "$pid" | wc -w)" -ne 1 ]; then
				continue
			fi
			daemon_pid["$d"]="$pid"

			sleep 3
		done
	done

	reboot=false
	while true ; do
		for d ; do
			if ! pidof "$d" | grep -q "\\<${daemon_pid[$d]}\\>" ; then
				crash_daemon="$d"
				reboot=true
				break
			fi
		done

		if $reboot ; then
			ha_log err "$crash_daemon crashed, reboot myself"
			return
		fi

		sleep 10
	done
}

trap 'pkill -P $$; exit 0' SIGTERM

watcher=(watch_both_passive watch_opensource_daemon)

for i in "${watcher[@]}"; do
    ha_log notice "Start [$i]"
	$i &
done

wait -n

sync; sync; sync;
reboot

